import pathlib
import contextlib
import sys
import appdirs
import requests
import datetime
from logging import getLogger
from typing import *
import http
import atcoder_submit_status.__about__ as version
import atcoder_submit_status.service as service
import atcoder_submit_status.utils as utils
from colorama import Fore, Back, Style

logger = getLogger(__name__)

USER_DATA_PATH = pathlib.Path(appdirs.user_data_dir('atcoder-submit-status'))
DEFAULT_COOKIE_PATH = USER_DATA_PATH / 'cookie.jar'
def get_cookie_path(service: service.Service):
   return USER_DATA_PATH / service.get_name() / 'cookie.jar'

CHECK = 'CHECK: '
ADD = 'ADD: '
NETWORK = 'NETWORK: '
DISPLAY = 'DISPLAY: '
FAILURE = 'FAILURE: '
HINT = 'HINT: '
INPUT = 'INPUT: '
SUCCESS = 'SUCCESS: '
INPUT_ICON = '[' + Fore.YELLOW + '>' + Style.RESET_ALL + '] '

def foreRGB(r: int, g: int, b: int) -> str:
   r = '{:03}'.format(r)
   g = '{:03}'.format(g)
   b = '{:03}'.format(b)
   return "\x1b[38;2;" + r + ";" + g + ";" + b + "m"

def backRGB(r: int, g: int, b: int) -> str:
   r = '{:03}'.format(r)
   g = '{:03}'.format(g)
   b = '{:03}'.format(b)
   return "\x1b[48;2;" + r + ";" + g + ";" + b + "m"

def delete_lines(n_lines: int):
   for _ in range(n_lines):
      print(f'\x1b[1T', end='')

def convert_timestamp_with_time_zone_to_date(timestamp: str) -> str:
   date = datetime.datetime.strptime(timestamp, '%Y-%m-%d %H:%M:%S%z')
   return date.strftime('%Y-%m-%d %H:%M:%S')

def convert_language_with_version_to_language(lang: str) -> str:
   return lang[:lang.find(' ')]

def get_task_id(full_task_name: str) -> str:
   return full_task_name[:full_task_name.find(' ')]


@contextlib.contextmanager
def new_session_with_our_user_agent(cookie_path: pathlib.Path, service: service.Service=None) -> Iterator[requests.Session]:
   if service is not None:
      cookie_path = get_cookie_path(service)
   session = requests.Session()
   session.headers['User-Agent'] = f'{version.__package_name__}/{version.__version__} (+{version.__url__})'
   logger.debug(f'User-Agent: {session.headers["User-Agent"]}')
   try:
      with utils.with_cookiejar(session, path=cookie_path) as session:
         yield session
   except Exception as e:
      logger.info(utils.HINT + f'You can delete the broken cookie.jar file: {str(cookie_path)}')
      raise

def service_from_url(url: str) -> service.Service:
   """
   コンテストURLからサービスを取得します。
   """
   if 'atcoder' in url:
      return service.AtCoderService()
   else:
      logger.info('service name could not be found.')
      return None

_DEFAULT_SESSION = None
def get_default_session() -> requests.Session:
    global _DEFAULT_SESSION
    if _DEFAULT_SESSION is None:
        _DEFAULT_SESSION = requests.session()
    return _DEFAULT_SESSION

@contextlib.contextmanager
def with_cookiejar(session: requests.Session, *, path: pathlib.Path=DEFAULT_COOKIE_PATH) -> Iterator[requests.Session]:
   """Cookieを利用したセッション
   """
   session.cookies = http.cookiejar.LWPCookieJar(str(path))  # type: ignore
   if path.exists():
      logger.info('load cookie from: %s', path)
      session.cookies.load(ignore_discard=True)  # type: ignore
   yield session
   logger.debug('save cookie to: %s', path)
   path.parent.mkdir(parents=True, exist_ok=True)
   session.cookies.save(ignore_discard=True)  # type: ignore
   path.chmod(0o600)  # NOTE: to make secure a little bit



import argparse
import getpass
from random import choices
import time
from logging import getLogger

import atcoder_submit_status.utils as utils

logger = getLogger(__name__)

def add_subparser(subparsers: argparse.Action) -> None:
   subparsers_add_parser: Callable[..., argparse.ArgumentParser] = subparsers.add_parser  # type: ignore
   subparser = subparsers_add_parser('login', aliases=['l'], help='login to a service', formatter_class=argparse.RawTextHelpFormatter, epilog='''\
Supported Services:
  √ AtCoder
''')
   subparser.add_argument('service', choices=['atcoder'])
   subparser.add_argument('-u', '--username')
   subparser.add_argument('-p', '--password')
   subparser.add_argument('--check', action='store_true', help='check whether you are logged in or not')

def run(args: argparse.Namespace) -> bool:
   service = utils.service_from_url(args.service)

   if service is None:
      return False

   with utils.new_session_with_our_user_agent(args.cookie, service=service) as session:
      if service.is_logged_in(session):
         logger.info(utils.SUCCESS + 'You have already signed in.')
         return True
      else:
         logger.info(utils.FAILURE + 'You are not signed in.')
         if args.check:
            return False

      username = args.username
      if username is None:
         username = input(utils.INPUT_ICON + 'Username: ')
      password = args.password
      if password is None:
         password = getpass.getpass(prompt=f'{utils.INPUT_ICON}Password: ')
      time.sleep(1)

      service.login(username, password, session)
      time.sleep(1)
      if service.is_logged_in(session):
         logger.info(utils.SUCCESS + 'You have already signed in.')
      else:
         logger.info(utils.FAILURE + 'You are not signed in.')



from abc import abstractmethod
from datetime import datetime
import sys
import time
from copy import deepcopy
import re
from typing import *
import requests
from bs4 import BeautifulSoup
from colorama import Fore, Back, Style
import atcoder_submit_status.__about__ as version
import atcoder_submit_status.utils as utils
from rich.table import Table
from logging import getLogger
logger = getLogger(__name__)


class Service:
   @abstractmethod
   def get_login_page_url(self) -> str:
      pass


   @abstractmethod
   def login(self, username, password, session):
      pass


   @abstractmethod
   def is_logged_in(self) -> bool:
      pass


   @abstractmethod
   def fetch_submissions(self, url, tasks, languages, statuses, users, session):
      pass


   @abstractmethod
   def minimize_submissions_info(self, submissions, mode):
      pass


   @abstractmethod
   def make_drawable_submissions(self, submissions, no_color):
      pass


   @abstractmethod
   def get_tasks(self, tasks_url, session: Optional[requests.Session] = None):
      pass


   @abstractmethod
   def get_url(self) -> str:
      pass


   @abstractmethod
   def get_name(self) -> str:
      pass


class AtCoderService(Service):
   def get_login_page_url(self) -> str:
      return 'https://atcoder.jp/login'


   def login(self, username: str, password: str, session: Optional[requests.Session] = None):
      session = session or utils.get_default_session()
      if self.is_logged_in(session=session):
         return
      url = self.get_login_page_url()
      logger.info(utils.NETWORK + f'GET: {url}')
      response = session.get(url)
      soup = BeautifulSoup(response.text, 'lxml')
      csrf_token = soup.find(attrs={'name': 'csrf_token'}).get('value')
      login_info = { 'csrf_token': csrf_token, 'username': username, 'password': password }
      try:
         response = session.post(url, data=login_info)
         response.raise_for_status()
      except requests.exceptions.HTTPError as e:
         logger.error(e)
         sys.exit(0)



   def is_logged_in(self, session: Optional[requests.Session] = None) -> bool:
      session = session or utils.get_default_session()
      url = 'https://atcoder.jp/contests/dummydummydummy/submit'
      logger.debug(utils.NETWORK + f'GET: {url}')
      response = session.get(url)
      if response.status_code == 404:
         return True
      else:
         return False


   def fetch_submissions(self, url: str, tasks: List[str] = [], languages: List[str] = [], statuses: List[str] = [], users: List[str] = [], session: Optional[requests.Session] = None):
      session = session or utils.get_default_session()

      contest_round = self.get_round(url)
      submissions_url = self.get_url() + '/contests/' + contest_round + '/submissions'

      # 全体の提出を見る権限があるかを確認
      response = session.get(submissions_url)
      logger.debug(f'status_code = {response.status_code}')
      logger.debug(f'type(status_code) = {type(response.status_code)}')
      if response.status_code == 404:
         submissions_url += '/me'

      # フィルタ用データの調整
      if not users:
         name = self._get_user_name()
         if name:
            users.append(name)
         else:
            logger.info(utils.FAILURE + 'users not found.')
            sys.exit(0)

      submissions = []
      keys = self._get_all_headers()
      for user in users:
         page = 0
         while True:  # page がなくなるまで
            page += 1
            time.sleep(0.25)

            payload = { 'page': page }
            if user:
               payload['f.User'] = user

            try:
               response = session.get(submissions_url, params=payload)
               response.raise_for_status()
            except requests.exceptions.HTTPError as e:
               logger.error(e)
               sys.exit(0)

            soup = BeautifulSoup(response.text, 'lxml')
            
            tables = soup.findAll('table', { 'class': 'table' }) 
            if not tables:
               break
            rows = tables[0].findAll('tr')

            for i in range(len(rows)):
               if i == 0:
                  continue

               # HTMLの内容をパース
               r = rows[i].findAll('td')
               submission = {}
               for i in range(len(keys)):
                  if i < len(r) - 1:  # "Detail" の分、1個引く
                     submission[keys[i]] = r[i].get_text().strip()
                  else:
                     submission[keys[i]] = ''

               # データを整形する
               submission['submission_time'] = utils.convert_timestamp_with_time_zone_to_date(submission['submission_time'])

               # フィルター
               if not tasks or utils.get_task_id(submission['task']).lower() in [t.lower() for t in tasks]:
                  if not statuses or submission['status'] in statuses or sum(int(submission['status'] not in self._get_statuses() and s == 'WJ') for s in statuses) != 0:
                     if not languages or utils.convert_language_with_version_to_language(submission['language']) in languages:
                        submissions.append(submission)

      submissions = sorted(submissions, key=lambda x: datetime.strptime(x['submission_time'], '%Y-%m-%d %H:%M:%S'))

      return submissions

   def minimize_submissions_info(self, submissions, mode):
      res = deepcopy(submissions)
      for i in range(len(res)):
         if 'submission_time' in res[i]:
            if mode == 'MINIMAL':
               res[i].pop('submission_time')
         if 'task' in res[i]:
            res[i]['task'] = utils.get_task_id(res[i]['task'])
         if 'language' in res[i]:
            if mode == 'MINIMAL':
               res[i].pop('language')
            elif mode == 'NORMAL':
               res[i]['language'] = utils.convert_language_with_version_to_language(res[i]['language'])
         if 'code_size' in res[i]:
            res[i].pop('code_size')
         if 'exec_time' in res[i]:
            res[i].pop('exec_time')
         if 'memory' in res[i]:
            res[i].pop('memory')

      return res


   def make_drawable_submissions(self, submissions, no_color: bool):
      submits = deepcopy(submissions)

      # Statusには色をつけておく
      from rich.padding import Padding
      for i in range(len(submits)):
         status = submits[i]['status'][-3:].strip()
         if not no_color:
            color = self._get_status_color(status)
            submits[i]['status'] = Padding(submits[i]['status'], (0, 1), style=color)

      # Tableを構築する
      table = Table()

      if len(submits) == 0:
         return table

      ## カラムの定義
      for key in submits[0].keys():
         if key in ['score', 'code_size', 'exec_time', 'memory']:
            table.add_column(key, justify='right')
         elif key in ['status']:
            table.add_column(key, justify='center')
         else:
            table.add_column(key, justify='left')

      ## ラインの定義
      for submit in submits:
         row = [val for val in submit.values()]
         table.add_row(*row)

      return table


   def get_task_names(self, tasks_url, session: Optional[requests.Session] = None) -> List[str]:
      session = session or utils.get_default_session()

      try:
         response = session.get(tasks_url)
         response.raise_for_status()
      except requests.exceptions.HTTPError as e:
         logger.error(e)
         sys.exit(0)

      soup = BeautifulSoup(response.text, 'lxml')
      rows = soup.findAll('table', {'class': 'table' })[0].findAll('tr')
      
      task_names = []
      for i in range(len(rows)):
         if i == 0:
            continue
         r = rows[i].findAll('td')
         task_name = f'{r[0].get_text()} - {r[1].get_text()}'
         task_names.append(task_name)

      return task_names


   def get_url(self):
      return 'https://atcoder.jp'


   def get_name(self):
      return 'atcoder'


   def get_round(self, url: str) -> str:
      """
      コンテスト名を取得します。

      url が 'https://atcoder.jp/contests/' で始まらないとき、url が round 名そのものであると解釈します。
      """
      prefix = 'https://atcoder.jp/contests/'

      if url.startswith(prefix):
         if url[-1] != '/':
            url = url + '/'
         return url[len(prefix):url.find('/', len(prefix))]
      else:
         return url


# private
   def _get_user_name(self) -> str:
      with open(utils.get_cookie_path(self)) as f:
         res = re.search(r'UserName%3A(.*?)%00', f.read())
         if res:
            s = res.group(1)
            return s
         else:
            return None


   def _get_all_headers(self) -> List[str]:
      return ['submission_time', 'task', 'user', 'language', 'score', 'code_size', 'status', 'exec_time', 'memory']


   def _get_statuses(self) -> List[str]:
      return ['AC', 'CE', 'MLE', 'TLE', 'RE', 'OLE', 'IE', 'WA', 'WJ', 'WR']


   def _get_status_color(self, status: str):
      green = ['AC']
      yellow = ['CE', 'MLE', 'TLE', 'RE', 'OLE', 'IE', 'WA']
      gray_status = ['WJ', 'WR']
      if status in green:
         return 'rgb(255,255,255) on rgb(92,184,92)'
      elif status in yellow:
         return 'rgb(255,255,255) on rgb(240,173,78)'
      else:
         return 'rgb(255,255,255) on rgb(153,153,153)'



import argparse
from ast import arg, parse
from distutils.log import ERROR
from email.errors import HeaderParseError
import sys
from typing import *
from logging import CRITICAL, WARN, DEBUG, INFO, StreamHandler, basicConfig, getLogger
import pathlib
import atcoder_submit_status.__about__ as version
import atcoder_submit_status.utils as utils
import atcoder_submit_status.login as subcommands_login
import atcoder_submit_status.logout as subcommands_logout
import atcoder_submit_status.watch as subcommands_watch
import atcoder_submit_status.fetch as subcommands_fetch
import atcoder_submit_status.log_formatter as log_formatter

logger = getLogger(__name__)

def get_parser() -> argparse.ArgumentParser:
   """パーサを生成します。
   """

   parser = argparse.ArgumentParser(
      description='Tools for Watching AtCoder Submissions',
      formatter_class=argparse.RawTextHelpFormatter,
      epilog='''\
Tips:
  README: https://github.com/ngng628/atcoder-submit-status/blob/main/README.md
'''
   )

   parser.add_argument('-v', '--verbose', action='store_true')
   parser.add_argument('-c', '--cookie', type=pathlib.Path, default=utils.DEFAULT_COOKIE_PATH, help=f'path to cookie. (default: {utils.DEFAULT_COOKIE_PATH})')
   parser.add_argument('-q', '--quiet', action='count', default=0, help='Give less output. Option is additive, and can be used up to 3 times.')
   parser.add_argument('--version', action='store_true', help='print the atcoder-submit-status version number.')

   subparsers = parser.add_subparsers(dest='subcommand', help=f'for details, see "{sys.argv[0]} COMMAND --help"')
   subcommands_login.add_subparser(subparsers)
   subcommands_logout.add_subparser(subparsers)
   subcommands_watch.add_subparser(subparsers)
   subcommands_fetch.add_subparser(subparsers)

   return parser

def run_program(args: argparse.Namespace, parser: argparse.ArgumentParser) -> int:
   if args.version:
      print('atcoder-submit-status {}'.format(version.__version__))
      return 0
   logger.debug(f'args: {str(args)}')

   logger.info('atcoder-submit-status %s', version.__version__)

   if args.subcommand in ['login', 'l']:
      if not subcommands_login.run(args):
         return 1
   elif args.subcommand in ['logout']:
      if not subcommands_logout.run(args):
         return 1
   elif args.subcommand in ['watch', 'w']:
      if not subcommands_watch.run(args):
         return 1
   elif args.subcommand in ['fetch', 'f']:
      if not subcommands_fetch.run(args):
         return 1
   else:
      parser.print_help(file=sys.stderr)
      return 1
   return 0

def main(args: Optional[List[str]] = None) -> 'NoReturn':
   parser = get_parser()
   parsed = parser.parse_args(args=args)

   level = INFO
   if parsed.verbose:
      level = DEBUG
   if parsed.quiet == 1:
      level = WARN
   elif parsed.quiet == 2:
      level = ERROR
   elif parsed.quiet == 3:
      level = CRITICAL

   handler = StreamHandler(sys.stdout)
   handler.setFormatter(log_formatter.LogFormatter())
   basicConfig(level=level, handlers=[handler])

   # is_updated = update_checking.run()

   try:
      sys.exit(run_program(parsed, parser=parser))
   except NotImplementedError as e:
      logger.error('NotImplementedError')
      sys.exit(1)
   except Exception as e:
      logger.exception(str(e))
      sys.exit(1)


if __name__ == '__main__':
   main()



import argparse
from copy import deepcopy
import sys
import sys
import time
from typing import Optional
import requests
from logging import getLogger
from colorama import Fore, Back, Style, Cursor
from rich.console import Console
from rich.live import Live

import atcoder_submit_status.utils as utils
import atcoder_submit_status.service as service

logger = getLogger(__name__)

def add_subparser(subparsers: argparse.Action) -> None:
   subparsers_add_parser: Callable[..., argparse.ArgumentParser] = subparsers.add_parser  # type: ignore
   subparser = subparsers_add_parser('watch', aliases=['w'], help='watch the contest submissions', formatter_class=  argparse.RawTextHelpFormatter, epilog='''\
Supported Services:
  √ AtCoder
''')
   subparser.add_argument('url', help='Contest URL (or AtCoder contest name)')
   subparser.add_argument('--no-color', action='store_true', help='Turn off color')
   subparser.add_argument('--tasks', metavar='<task-name>', default=[], nargs='*', help='Select tasks.\n(e.g. a b d ex)')
   subparser.add_argument('--languages', metavar='<lang>', default=[], nargs='*', help='Select languages.\n(e.g. C++ C#)')
   subparser.add_argument('--statuses', default=[], nargs='*', choices=['AC', 'CE', 'MLE', 'TLE', 'RE', 'OLE', 'IE', 'WA', 'WJ', 'WR'], help='Select statuses.\n(e.g. WA TLE)')
   subparser.add_argument('-u', '--users', metavar='<user-name>', default=[], nargs='*', help='Select users.')
   subparser.add_argument('--info-level', default='NORMAL', choices=['MINIMAL', 'NORMAL', 'DETAILS'], help='Select output information level.')
   subparser.add_argument('-r', '--reverse', action='store_true', help='Reverse submissions')
   subparser.add_argument('-t', '--tail', metavar='<n-lines>', default=sys.maxsize, type=int, help='Print the last <n-lines> submissions.')


def _fetch(args: argparse.Namespace, srv: service.Service, session: Optional[requests.Session] = None):
   session = session or utils.get_default_session()
   submissions = srv.fetch_submissions(args.url, tasks=args.tasks, languages=args.languages, statuses=args.statuses, users=args.users, session=session)
   if args.info_level != 'DETAILS':
      submissions = srv.minimize_submissions_info(submissions, args.info_level)
   return submissions


def run(args: argparse.Namespace) -> bool:
   logger.debug(f'users: {args.users}')
   srv = utils.service_from_url(args.url)

   if srv is None:
      logger.info('we predict that the service you use is AtCoder.')
      srv = service.AtCoderService()

   with utils.new_session_with_our_user_agent(args.cookie, service=srv) as session:
      logger.debug('start session')
      if not srv.is_logged_in(session):
         logger.info(utils.FAILURE + 'You are not signed in.')
         logger.info(utils.HINT + f'You can try to enter this command: `acss login {srv.get_name()}`')
         return False

      submissions = []
      try:
         with Live(refresh_per_second=1) as live:
            while True:
               submissions = srv.make_drawable_submissions(_fetch(args, srv=srv, session=session)[-args.tail:], args.no_color)
               if args.reverse:
                  submissions.reverse()
               live.update(submissions)
               time.sleep(1)
      except KeyboardInterrupt:
         sys.exit(0)



__package_name__ = 'atcoder-submit-status'
__author__ = 'ngng628'
__email__ = 'ngng628.kyopro@gmail.com'
__license__ = 'MIT License'
__url__ = 'https://github.com/ngng628/atcoder-submit-status'
__version_info__ = (0, 0, 4)
__version__ = '.'.join(map(str, __version_info__[:3]))
__description__ = 'CLI tool to fetch atcoder-submit-status'


import argparse
import sys
import codecs
import pathlib
import time
from typing import Optional
import requests
from logging import getLogger
from colorama import Fore, Back, Style

import atcoder_submit_status.utils as utils
import atcoder_submit_status.service as service

logger = getLogger(__name__)

def add_subparser(subparsers: argparse.Action) -> None:
   subparsers_add_parser: Callable[..., argparse.ArgumentParser] = subparsers.add_parser  # type: ignore
   subparser = subparsers_add_parser('fetch', aliases=['f'], help='Fetch the contest submissions', formatter_class=  argparse.RawTextHelpFormatter, epilog='''\
Supported Services:
  √ AtCoder
''')
   subparser.add_argument('url', help='Contest URL (or AtCoder contest name)')
   subparser.add_argument('-o', '--output-path',  metavar='<file>', type=pathlib.Path, help='Place the output into <file>.')
   subparser.add_argument('-S', '--separator', metavar='<sep>', type=str, default=',', help='Use <sep> instead of `,` for field separator.')
   subparser.add_argument('-e', '--encoding', metavar='<enc>', type=str, default='utf-8', help='Select charactor encoding.')
   subparser.add_argument('--tasks', metavar='<task-name>', default=[], nargs='*', help='Select tasks.\n(e.g. a b d ex)')
   subparser.add_argument('--languages', metavar='<lang>', default=[], nargs='*', help='Select languages.\n(e.g. C++ C#)')
   subparser.add_argument('--statuses', default=[], nargs='*', choices=['AC', 'CE', 'MLE', 'TLE', 'RE', 'OLE', 'IE', 'WA', 'WJ', 'WR'], help='Select statuses.\n(e.g. WA TLE)')
   subparser.add_argument('-u', '--users', metavar='<user-name>', default=[], nargs='*', help='Select users.')
   subparser.add_argument('--info-level', default='NORMAL', choices=['MINIMAL', 'NORMAL', 'DETAILS'], help='Select output information level.')
   subparser.add_argument('-r', '--reverse', action='store_true', help='Reverse submissions')
   subparser.add_argument('-t', '--tail', metavar='<n-lines>', default=sys.maxsize, type=int, help='Print the last <n-lines> submissions.')


def _fetch(args: argparse.Namespace, srv: service.Service, session: Optional[requests.Session] = None):
   session = session or utils.get_default_session()
   submissions = srv.fetch_submissions(args.url, tasks=args.tasks, languages=args.languages, statuses=args.statuses, users=args.users, session=session)
   if args.info_level != 'DETAILS':
      submissions = srv.minimize_submissions_info(submissions, args.info_level)
   return submissions


def run(args: argparse.Namespace) -> bool:
   logger.debug(f'users: {args.users}')
   srv = utils.service_from_url(args.url)

   if srv is None:
      logger.info('we predict that the service you use is AtCoder.')
      srv = service.AtCoderService()

   with utils.new_session_with_our_user_agent(args.cookie, service=srv) as session:
      logger.debug('start session')
      if not srv.is_logged_in(session):
         logger.info(utils.FAILURE + 'You are not signed in.')
         logger.info(utils.HINT + f'You can try to enter this command: `acss login {srv.get_name()}`')
         return False
      
      sep = codecs.decode(args.separator, 'unicode-escape')

      try:
         submissions = _fetch(args, srv=srv, session=session)
         file = sys.stdout if args.output_path is None else codecs.open(str(args.output_path), mode='w', encoding=args.encoding)
         for s in submissions:
            for i, v in enumerate(s.values()):
               if i != 0:
                  print(end=sep, file=file)
               print(v, end='', file=file)
            print(file=file)

         if args.output_path:
            logger.info(utils.SUCCESS + f'Write submissions to `{str(args.output_path)}`.')
         

      except KeyboardInterrupt:
         sys.exit(0)



import http.client
import logging
from typing import *

import colorama

colorama.init()

log_colors_level = {
    logging.DEBUG: '[' + colorama.Fore.RED + 'DEBUG' + colorama.Style.RESET_ALL + '] ',
    logging.INFO: '[' + colorama.Fore.BLUE + 'INFO' + colorama.Style.RESET_ALL + '] ',
    logging.WARNING: '[' + colorama.Fore.YELLOW + 'WARNING' + colorama.Style.RESET_ALL + '] ',
    logging.ERROR: '[' + colorama.Fore.RED + 'ERROR' + colorama.Style.RESET_ALL + '] ',
    logging.CRITICAL: '[' + colorama.Fore.RED + colorama.Style.BRIGHT + 'CRITICAL' + colorama.Style.RESET_ALL + '] ',
}

log_colors_semantics = {
    'NO_HEADER': '',
    'HINT': '[' + colorama.Fore.YELLOW + '?' + colorama.Style.RESET_ALL + '] ',
    'NETWORK': '[' + colorama.Fore.MAGENTA + 'NETWORK' + colorama.Style.RESET_ALL + '] ',
    'SUCCESS': '[' + colorama.Fore.GREEN + 'SUCCESS' + colorama.Style.RESET_ALL + '] ',
    'FAILURE': '[' + colorama.Fore.RED + 'FAILURE' + colorama.Style.RESET_ALL + '] ',
    'INPUT': '[' + colorama.Fore.YELLOW + 'INPUT' + colorama.Style.RESET_ALL + '] ',
}

status_code_messages: Set[str] = {str(int(key)) + ' ' + str(value) for key, value in http.client.responses.items()}


class LogFormatter(logging.Formatter):
    def __init__(self, datefmt: Optional[str] = None):
        fmt = '[%(levelname)s] %(message)s'
        super().__init__(fmt=fmt, datefmt=datefmt)

    def format(self, record: logging.LogRecord) -> str:
        if record.levelno not in log_colors_level:
            return super().format(record)

        # detect the heading from the record
        heading = None
        message = record.getMessage()
        if not message and record.exc_info is None:
            heading = ''
        if heading is None:
            for key, value in log_colors_semantics.items():
                if message.upper().startswith(key + ':'):
                    heading = value
                    message = message[len(key + ':'):].lstrip()
                    break
        if heading is None:
            heading = log_colors_level[record.levelno]

        # exception
        if record.exc_info is not None:
            message += '\n' + self.formatException(record.exc_info)

        # make a string
        if not heading:
            return message
        return heading + message



from setuptools import find_packages, setup

import atcoder_submit_status.__about__ as version

with open('README.md', 'r') as fp:
    readme = fp.read()

setup(
    name=version.__package_name__,
    version=version.__version__,
    author=version.__author__,
    author_email=version.__email__,
    url=version.__url__,
    license=version.__license__,
    keywords="atcoder",
    description=version.__description__,
    long_description=readme,
    python_requires='>=3.6',
    install_requires=[name.rstrip() for name in open('requirements.txt').readlines()],
    packages=find_packages(),
    entry_points={
        'console_scripts': [
            'acss = atcoder_submit_status.main:main',
        ],
    },
    classifiers={
        'Programming Language :: Python :: 3',
        'Environment :: Console',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Utilities',
        'License :: OSI Approved :: MIT License'
    }
)



import os
import argparse
import getpass
from logging import getLogger
import pathlib

import atcoder_submit_status.utils as utils

logger = getLogger(__name__)

def add_subparser(subparsers: argparse.Action) -> None:
   subparsers_add_parser: Callable[..., argparse.ArgumentParser] = subparsers.add_parser  # type: ignore
   subparser = subparsers_add_parser('logout', help='logout to a service', formatter_class=argparse.RawTextHelpFormatter, epilog='''\
Supported Services:
  √ AtCoder
''')
   subparser.add_argument('service', choices=['atcoder'])
   subparser.add_argument('--check', action='store_true', help='check whether you are logged out or not')

def run(args: argparse.Namespace) -> bool:
   service = utils.service_from_url(args.service)

   if service is None:
      return False
   
   cookie_path = utils.get_cookie_path(service=service)
   if os.path.exists(cookie_path):
      os.remove(cookie_path)
      logger.info(f'delete cookie to {cookie_path}')
      logger.info(utils.SUCCESS + 'You have already signed out.')
      return True
   else:
      logger.info(utils.FAILURE + 'You are not signed in.')
      if args.check:
         return False
      return True
      